
local WindowBackdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 3, right = 3, top = 3, bottom = 3 }
}

local PaneBackdrop = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 3, right = 3, top = 5, bottom = 3 }
}

local DraggerBackdrop  = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = nil,
	tile = true, tileSize = 16, edgeSize = 0,
	insets = { left = 3, right = 3, top = 7, bottom = 7 }
}



function RLNote:Show()
  if not self.frame then
    local frame = CreateFrame("Frame", "RLNoteFrame", UIParent)

    frame:SetWidth(self.settings.display.w)
    frame:SetHeight(self.settings.display.h)
    frame:SetPoint("CENTER", UIParent, "CENTER", self.settings.display.x, self.settings.display.y)
    frame:SetBackdrop(WindowBackdrop)
    frame:SetBackdropColor(0, 0, 0, 1)
    frame:SetBackdropBorderColor(0.4, 0.4, 0.4)

    frame:EnableMouse()
    frame:SetMovable(true)
    frame:SetResizable(true)
    frame:SetClampedToScreen(true)
    frame:SetFrameStrata("DIALOG")

    -- title bar
    local titlebar = frame:CreateTexture(nil, "BACKGROUND")
    titlebar:SetTexture(0.5, 0.5, 0.5)
    titlebar:SetGradient("HORIZONTAL", 0.6, 0.6, 0.6, 0.3, 0.3, 0.3)
    titlebar:SetPoint("TOPLEFT", 4, -4)
    titlebar:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", -4, -28)

    local titlebarframe = CreateFrame("Frame", nil, frame)
    titlebarframe:SetAllPoints(titlebar)
    titlebarframe:EnableMouse()

    local titletext = frame:CreateFontString(nil, "ARTWORK")
    titletext:SetFontObject(GameFontNormal)
    titletext:SetTextColor(0.6, 0.6, 0.6)
    titletext:SetPoint("TOPLEFT", titlebar, "TOPLEFT", 26, 0)
    titletext:SetPoint("BOTTOMRIGHT", titlebar)

    titletext:SetHeight(28)
    titletext:SetText("RLNote")
    titletext:SetJustifyH("LEFT")
    titletext:SetJustifyV("MIDDLE")

    -- close button
    local close = CreateFrame("Button", nil, frame, "UIPanelCloseButton")
    close:SetPoint("TOPRIGHT", 0, 0)
    close:SetFrameLevel(3)
    close:SetScript("OnClick", function() frame:Hide() end)

    -- resizer
    local sizer_se = CreateFrame("Frame", nil, frame)
    sizer_se:SetPoint("BOTTOMRIGHT", -3, 3)
    sizer_se:SetWidth(16)
    sizer_se:SetHeight(16)

    local sizer_se_tex = frame:CreateTexture(nil, "BORDER")
    sizer_se_tex:SetTexture([[Interface\AddOns\RLNote\Textures\resize.tga]])
    sizer_se_tex:SetVertexColor(0.4, 0.4, 0.4, 1)
    sizer_se_tex:SetAllPoints(sizer_se)

    local function save_frame_rect()
      local _, _, sw, sh = UIParent:GetRect()
      local x, y, w, h = frame:GetRect()
      self.settings.display.x, self.settings.display.y, self.settings.display.w, self.settings.display.h =
      x + w/2 - sw/2, y + h/2 - sh/2, w, h
    end

    titlebarframe:SetScript("OnMouseDown", function()
      frame:StartMoving()
    end)
    titlebarframe:SetScript("OnMouseUp", function()
      frame:StopMovingOrSizing()
      save_frame_rect()
    end)

    sizer_se:SetScript("OnEnter", function()
      sizer_se_tex:SetVertexColor(0.8, 0.8, 0.8, 1)
    end)

    sizer_se:SetScript("OnLeave", function()
      sizer_se_tex:SetVertexColor(0.4, 0.4, 0.4, 1)
    end)

    sizer_se:SetScript("OnMouseDown", function()
      frame:SetMinResize(600, 270)
      frame:SetMaxResize(2000, 2000)
      frame:StartSizing()
    end)

    sizer_se:SetScript("OnMouseUp", function()
      frame:StopMovingOrSizing()
      save_frame_rect()
    end)


    -- notelist
    local notelist_border = CreateFrame("Frame", nil, frame)
    notelist_border:SetPoint("TOPLEFT", frame, "TOPLEFT", 10, -30)
    notelist_border:SetPoint("BOTTOMRIGHT", frame, "BOTTOMLEFT", self.settings.display.notelist_width, 10)

    notelist_border:SetBackdrop(PaneBackdrop)
    notelist_border:SetBackdropColor(0.1, 0.1, 0.1, 0.5)
    notelist_border:SetBackdropBorderColor(0.4, 0.4, 0.4)
    notelist_border:SetResizable(true)
    notelist_border:SetMinResize(50, 50)
    notelist_border:SetMaxResize(2000, 2000)

    local notelist = CreateFrame("ScrollFrame", nil, notelist_border)
    notelist:SetPoint("TOPLEFT", 8, -8)
    notelist:SetPoint("BOTTOMRIGHT", -8, 8)
    notelist:EnableMouseWheel(true)

    -- notelist scrollbar
    local notelist_scroll = CreateFrame("Slider", nil, notelist, "UIPanelScrollBarTemplate")
    notelist_scroll:SetPoint("BOTTOMRIGHT", notelist_border, "BOTTOMRIGHT", -8, 24)
    notelist_scroll:SetPoint("TOPRIGHT", notelist_border, "TOPRIGHT", -8, -24)
    notelist_scroll:SetMinMaxValues(0, 1000)
    notelist_scroll:SetValueStep(1)
    notelist_scroll:SetValue(0)
    notelist_scroll:SetWidth(16)
    notelist_scroll:SetFrameLevel(6)
    notelist_scroll:Hide()

    local notelist_scrollbg = notelist_scroll:CreateTexture(nil, "BACKGROUND")
    notelist_scrollbg:SetAllPoints()
    notelist_scrollbg:SetTexture(0, 0, 0, 1)

    local notelist_content = CreateFrame("Frame", nil, notelist)
    notelist:SetScrollChild(notelist_content)
    notelist_content:SetPoint("TOPLEFT")
    notelist_content:SetPoint("TOPRIGHT")

    notelist:SetScript("OnMouseWheel", function(frame, value)
      local l, h = notelist_scroll:GetMinMaxValues()
      notelist_scroll:SetValue(min(max(notelist_scroll:GetValue() - value * 45, l), h))
    end)

    notelist_scroll:SetScript("OnValueChanged", function(frame, value)
      notelist_content:SetPoint("TOPLEFT", 0, value)
      notelist_content:SetPoint("TOPRIGHT", 0, value)
    end)

    self.notelist_border = notelist_border
    self.notelist = notelist
    self.notelist_content = notelist_content
    self.notelist_scroll = notelist_scroll

    notelist:SetScript("OnSizeChanged", function() self:NoteList_SizeChanged() end)

    --Dragger
    local dragger = CreateFrame("Frame", nil, notelist_border)
    dragger:SetWidth(8)
    dragger:SetPoint("TOP", notelist_border, "TOPRIGHT")
    dragger:SetPoint("BOTTOM", notelist_border, "BOTTOMRIGHT")
    dragger:SetBackdrop(DraggerBackDrop)
    dragger:SetBackdropColor(1, 1, 1, 0)
    dragger:EnableMouse(true)

    local function Dragger_OnLeave(frame)
      frame:SetBackdropColor(1, 1, 1, 0)
    end

    local function Dragger_OnEnter(frame)
      frame:SetBackdropColor(1, 1, 1, 0.8)
    end

    local function Dragger_OnMouseDown(frame)
      notelist_border:StartSizing("RIGHT")
    end

    local function Dragger_OnMouseUp()
      notelist_border:StopMovingSizing()
      notelist_border:SetUserPlaced(false)

      local w = notelist_border:GetWidth() + 10
      notelist_border:ClearAllPoints()
      notelist_border:SetPoint("TOPLEFT", frame, "TOPLEFT", 10, -30)
      notelist_border:SetPoint("BOTTOMRIGHT", frame, "BOTTOMLEFT", w, 10)
      self.settings.display.notelist_width = w
    end

    dragger:SetScript("OnEnter", Dragger_OnEnter)
    dragger:SetScript("OnLeave", Dragger_OnLeave)
    dragger:SetScript("OnMouseDown", Dragger_OnMouseDown)
    dragger:SetScript("OnMouseUp", Dragger_OnMouseUp)


    -- note frame
    local noteframe_border = CreateFrame("Frame", nil, frame)
    noteframe_border:SetPoint("TOPLEFT", notelist_border, "TOPRIGHT")
    noteframe_border:SetPoint("BOTTOM", notelist_border, "BOTTOM")
    noteframe_border:SetPoint("RIGHT", frame, "RIGHT", -10, 0)

    noteframe_border:SetBackdrop(PanelBackdrop)
    noteframe_border:SetBackdropColor(0.1, 0.1, 0.1, 0.5)
    noteframe_border:SetBackdropBorderColor(0.4, 0.4, 0.4)

    local noteframe = CreateFrame("ScrollFrame", nil, noteframe_border)
    noteframe:SetPoint("TOPLEFT", 8, -8)
    noteframe:SetPoint("BOTTOMRIGHT", -8, 8)
    noteframe:EnableMouseWheel(true)

    local noteframe_scroll = CreateFrame("Slider", nil, name_list, "UIPanelScrollBarTemplate")
    noteframe_scroll:SetPoint("TOPRIGHT", 0, -16)
    noteframe_scroll:SetPoint("BOTTOMRIGHT", 0, 16)
    noteframe_scroll:SetMinMaxValues(0, 0)
    noteframe_scroll:SetValueStep(1)
    noteframe_scroll:SetValue(0)
    noteframe_scroll:SetWidth(16)
    noteframe_scroll:Hide();

    local noteframe_scrollbg = noteframe_scroll:CreateTexture(nil, "BACKGROUND")
    noteframe_scrollbg:SetAllPoints()
    noteframe_scrollbg:SetTexture(0, 0, 0, 1)

    local noteframe_content = CreateFrame("Frame", nil, noteframe)
    noteframe:SetScrollChild(noteframe_content)
    noteframe_content:SetPoint("TOPLEFT")
    noteframe_content:SetPoint("TOPRIGHT", -16, 0)

    noteframe:SetScript("OnMouseWheel", function(frame, value)
      local l, h = noteframe_scroll:GetMinMaxValues()
      noteframe_scroll:SetValue(min(max(noteframe_scroll:GetValue() - value * 36, l), h))
    end)

    noteframe_scroll:SetScript("OnValueChanged", function(frame, value)
      noteframe_content:SetPoint("TOPLEFT", 0, value)
    end)


    self.frame = frame
  end
  self.frame:Show()
end



------------
-- NoteList
------------
function RLNote:NoteList_SizeChanged()
  local ch = self.notelist_content:GetHeight()
  local h = self.notelist:GetHeight()

  if h >= ch then
    self.notelist:SetPoint("BOTTOMRIGHT", -8, 8)
    self.notelist_scroll:SetMinMaxValues(0, 0)
    self.notelist_scroll:Hide()
  else
    self.notelist:SetPoint("BOTTOMRIGHT", -8, 8)
    self.notelist_scroll:SetMinMaxValues(0, ch - h)
    self.notelist_scroll:Show()
  end
end
